#ifndef DetectorConstruction_h
#define DetectorConstruction_h 1

#include "G4VUserDetectorConstruction.hh"
#include "globals.hh"
#include "G4Material.hh"
#include "G4ThreeVector.hh"
#include "G4VSensitiveDetector.hh"

class G4LogicalVolume;
class DetectorMessenger;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class DetectorConstruction : public G4VUserDetectorConstruction
{
public:
  DetectorConstruction();
  virtual ~DetectorConstruction();

  virtual G4VPhysicalVolume* Construct();

  void SetWorldMaterial(const G4String&);
  void SetModuleMaterial(const G4String&);
  void SetModuleNumber(G4int val);
  void SetModuleMotherNumber(G4int val);
  void SetModuleSizeX(G4double val);
  void SetModuleSizeY(G4double val);
  void SetModuleSizeZ(G4double val);
  void SetModuleGap(G4double val);
  void SetModuleRotation(G4int val);
  void SetDetectorPosX(G4double val);
  void SetDetectorPosY(G4double val);
  void SetDetectorPosZ(G4double val);

  static G4int GetModuleNumber(){return moduleNumber;}
  static G4int GetModuleMotherNumber(){return moduleMotherNumber;}

  void UpdateGeometry();

private:
  DetectorConstruction & operator=(const DetectorConstruction &right);
  DetectorConstruction(const DetectorConstruction&);

  // user paramater
  static G4int moduleNumber;
  static G4int moduleMotherNumber;
  G4ThreeVector moduleSize;
  G4double moduleGap;
  G4int moduleRotation;
  G4ThreeVector detectorPos;

  G4Material*  moduleMaterial;
  G4Material*  moduleMotherMaterial;
  G4Material*  moduleGrandMotherMaterial;
  G4Material*  worldMaterial;

  G4LogicalVolume* logicModule;
  G4LogicalVolume* logicModuleMother;
  G4LogicalVolume* logicModuleGrandMother;
  G4LogicalVolume* logicWorld;

  G4VSensitiveDetector* himeSD;

  DetectorMessenger* detectorMessenger;
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#endif

