//
#include "BeamMessenger.hh"

#include "PrimaryGeneratorAction.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithADouble.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

BeamMessenger::BeamMessenger(PrimaryGeneratorAction* Pri):Primary(Pri)
{
  gunDir = new G4UIdirectory("/gun/state/");
  gunDir->SetGuidance("Modification Commands for Unbound Nucleus State");

  erelCmd = new G4UIcmdWithADoubleAndUnit("/gun/state/erel",this);
  erelCmd->SetGuidance("Set Relative Energy");
  erelCmd->SetParameterName("relative_energy",false);
  erelCmd->SetUnitCategory("Energy");
  erelCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  ratio2mCmd = new G4UIcmdWithADouble("/gun/state/ratio2m",this);
  ratio2mCmd->SetGuidance("Set Ratio for 2-");
  ratio2mCmd->SetParameterName("ratio2m",false);
  ratio2mCmd->SetRange("ratio2m>=0 && ratio2m<=1");
  ratio2mCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  ratio1mCmd = new G4UIcmdWithADouble("/gun/state/ratio1m",this);
  ratio1mCmd->SetGuidance("Set Ratio for 1-");
  ratio1mCmd->SetParameterName("ratio1m",false);
  ratio1mCmd->SetRange("ratio1m>=0 && ratio1m<=1");
  ratio1mCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  ratio1pCmd = new G4UIcmdWithADouble("/gun/state/ratio1p",this);
  ratio1pCmd->SetGuidance("Set Ratio for 1+");
  ratio1pCmd->SetParameterName("ratio1p",false);
  ratio1pCmd->SetRange("ratio1p>=0 && ratio1p<=1");
  ratio1pCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  ratio2pCmd = new G4UIcmdWithADouble("/gun/state/ratio2p",this);
  ratio2pCmd->SetGuidance("Set Ratio for 2+");
  ratio2pCmd->SetParameterName("ratio2p",false);
  ratio2pCmd->SetRange("ratio2p>=0 && ratio2p<=1");
  ratio2pCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

BeamMessenger::~BeamMessenger()
{
  delete gunDir;
  delete erelCmd;
  delete ratio2mCmd;
  delete ratio1mCmd;
  delete ratio1pCmd;
  delete ratio2pCmd;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void BeamMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
  if(command == erelCmd){
    Primary->SetErel(erelCmd->GetNewDoubleValue(newValue));
  }else if(command == ratio2mCmd){
    Primary->SetRatio2m(ratio2mCmd->GetNewDoubleValue(newValue));
  }else if(command == ratio1mCmd){
    Primary->SetRatio1m(ratio1mCmd->GetNewDoubleValue(newValue));
  }else if(command == ratio1pCmd){
    Primary->SetRatio1p(ratio1pCmd->GetNewDoubleValue(newValue));
  }else if(command == ratio2pCmd){
    Primary->SetRatio2p(ratio2pCmd->GetNewDoubleValue(newValue));
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

