// $Id: HimeSD.cc,v 1.1.1.1 2004/10/06 05:36:32 iwai Exp $
#include "DetectorConstruction.hh"
#include "HimeSD.hh"

#include "G4HCofThisEvent.hh"
#include "G4SDManager.hh"
#include "G4VPhysicalVolume.hh"
#include "G4Step.hh"
#include "G4Track.hh"
#include "G4VTouchable.hh"
#include "G4TouchableHistory.hh"
#include "G4VProcess.hh"

#include "TVector3.h"

#include "NMadoka.hh"
#include "CDataHimeSim.hh"

HimeSD::HimeSD(G4String name)
  : G4VSensitiveDetector(name)
{;}

HimeSD::~HimeSD()
{;}

void HimeSD::Initialize(G4HCofThisEvent* /* HCE */)
{;}

void HimeSD::EndOfEvent(G4HCofThisEvent*)
{;}

G4bool HimeSD::ProcessHits(G4Step* aStep, G4TouchableHistory* /*ROhist*/)
{
  G4StepPoint* preStepPoint = aStep->GetPreStepPoint();
  G4StepPoint* postStepPoint = aStep->GetPostStepPoint();
  Double_t hitTime = preStepPoint->GetGlobalTime();
  Double_t hitEnergyDeposit = aStep->GetTotalEnergyDeposit();
  Double_t hitKineticEnergy = preStepPoint->GetKineticEnergy();
  static TVector3 hitPosition;
  hitPosition.SetX(preStepPoint->GetPosition().x());
  hitPosition.SetY(preStepPoint->GetPosition().y());
  hitPosition.SetZ(preStepPoint->GetPosition().z());
  TVector3 hitMomentum;
  hitMomentum.SetX(preStepPoint->GetMomentum().x());
  hitMomentum.SetY(preStepPoint->GetMomentum().y());
  hitMomentum.SetZ(preStepPoint->GetMomentum().z());

  G4ParticleDefinition* pd = aStep->GetTrack()->GetDefinition();
  TString particleName(pd->GetParticleName().data());

  G4TouchableHistory* theTouchable = (G4TouchableHistory*)(preStepPoint->GetTouchable());

  G4int depth;//0:self, 1:mother, 2:grandmother, ....
  Int_t copyNo = theTouchable->GetCopyNumber(depth=0);
  Int_t copyNoMother = theTouchable->GetCopyNumber(depth=1);

  CDataHimeSim* dataHime = (CDataHimeSim*)NMadoka::FindObject("Nadeko");

  const G4VProcess* process = postStepPoint->GetProcessDefinedStep();
  TString processName;
  if(process) processName = process->GetProcessName().data();

  if(hitEnergyDeposit>0.){
    CDataHimeSim::OneData newonedata(copyNoMother, copyNo,
				     particleName, processName, hitTime/ns,
				     hitEnergyDeposit/MeV, hitKineticEnergy/MeV,
				     1./cm*hitPosition, 1./MeV*hitMomentum);
    dataHime->fDequeOfOneData.push_back(newonedata);

    //newonedata.Print();
  }
  
  return true;
}
