//
#include "PrimaryGeneratorAction.hh"
#include "BeamMessenger.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleGun.hh"

#include "Randomize.hh"

#include "NMadoka.hh"
#include "CDataHimeSim.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "CKinema.hh"
#include "FFunction.hh"

#include "TCanvas.h"
#include "TRandom3.h"
#include "TF1.h"

#include <fstream>
#include <cstdlib>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::PrimaryGeneratorAction()
  : p1(CParticleDefinition::GetParticle("neutron")),
    p2(CParticleDefinition::GetParticle("9Li")),
    //    p2(CParticleDefinition::GetParticle("10Be")),
    erel(1.*MeV),ratio2m(0),ratio1m(0),ratio1p(0),ratio2p(0)
{
  beamMessenger = new BeamMessenger(this);
  particleGun = new G4ParticleGun(1);

  random = new TRandom3(126);//static必須//seedをセットしないとGetRandomとかぶる

  f2m = new TF1("f2m", VirtualResonance, 0, 3, 5);
  f1m = new TF1("f1m", VirtualResonance, 0, 3, 5);
  f1p = new TF1("f1p", BrietWigner, 0, 3, 2);
  f2p = new TF1("f2p", BrietWigner, 0, 3, 2);

  f2m->SetParameters(-12.7, 3.0, 0.325/2., p1.m(), p2.m());
  f1m->SetParameters(-5.6, 3.0, 0.325/2, p1.m(), p2.m());
  f1p->SetParameters(0.22,0.09);
  f2p->SetParameters(0.53,0.45);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::~PrimaryGeneratorAction()
{
  delete f2p;
  delete f1p;
  delete f1m;
  delete f2m;

  delete random;

  delete particleGun;
  delete beamMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
  G4ThreeVector momentum = CalculateMomentum();
  G4ThreeVector position = G4ThreeVector(0., 0., 0.);

  G4ParticleDefinition* particle = particleGun->GetParticleDefinition();
  G4double mass = particle->GetPDGMass();
  G4double energy = sqrt(mass*mass+momentum*momentum) - mass;

  particleGun->SetParticleMomentumDirection(momentum);
  particleGun->SetParticleEnergy(energy);
  particleGun->SetParticlePosition(position);

  particleGun->GeneratePrimaryVertex(anEvent);
}

G4ThreeVector PrimaryGeneratorAction::CalculateMomentum()
{
  // relative enrgy spectrum
  // use erel as flag
  double erelnow;
  if(erel < 0){
    double divider = random->Uniform();
    divider -= ratio2m;
    if(divider < 0){
      erelnow = f2m->GetRandom();
    }else{
      divider -= ratio1m;
      if(divider < 0){
	erelnow = f1m->GetRandom();
      }else{
	divider -= ratio1p;
	if(divider < 0){
	  erelnow = f1p->GetRandom();	
	}else{
	  erelnow = f2p->GetRandom();	
	}
      }
    }
  }else{
    erelnow = erel;
  }

  static G4double kin(particleGun->GetParticleEnergy());//static必須
  CKinema::FissionErel(p1, p2, erelnow, kin);

  G4ThreeVector momentum = G4ThreeVector(p1.p(0)*MeV, p1.p(1)*MeV, p1.p(2)*MeV);

  CDataHimeSim* dataHime = (CDataHimeSim*)NMadoka::FindObject("Nadeko");
  dataHime->fMomentum = TVector3(p1.p(0), p1.p(1), p1.p(2));
  dataHime->fMomentumChargedParticle = TVector3(p2.p(0), p2.p(1), p2.p(2));

  return momentum;
}

void PrimaryGeneratorAction::SetErel(G4double val)
{
  erel = val;
}

void PrimaryGeneratorAction::SetRatio2m(G4double val)
{
  ratio2m = val;
}

void PrimaryGeneratorAction::SetRatio1m(G4double val)
{
  ratio1m = val;
}

void PrimaryGeneratorAction::SetRatio1p(G4double val)
{
  ratio1p = val;
}

void PrimaryGeneratorAction::SetRatio2p(G4double val)
{
  ratio2p = val;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
