//
#include "RunAction.hh"
#include "RunMessenger.hh"
#include "G4UImanager.hh"
#include "G4VVisManager.hh"
#include "G4Run.hh"
#include "globals.hh"

#include "NMadoka.hh"
#include "CDataHimeSim.hh"

#include <iomanip>
#include <sstream>

RunAction::RunAction() : fileName("run")
{
  runMessenger = new RunMessenger(this);
}

RunAction::~RunAction()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void RunAction::BeginOfRunAction(const G4Run* aRun)
{
  G4int id = aRun->GetRunID();
  G4cout << "### Run " << id << " start" << G4endl;

  std::ostringstream oss;
  oss << "root/" << fileName
      << std::setw(4) << std::setfill('0') << id << ".root";
  NMadoka::OpenFile(oss.str(), "recreate");
  NMadoka::CreateTree("Koyomi", "experiment tree");
  CDataHimeSim* datahime = new CDataHimeSim("Nadeko");//do not delete
  // NMadoka::CreateBranch("Koyomi", "hime", datahime, 32000, 0);
  NMadoka::CreateBranch("Koyomi", "hime", datahime);

#ifdef G4VIS_USE
  G4UImanager* UI = G4UImanager::GetUIpointer();

  G4VVisManager* pVVisManager = G4VVisManager::GetConcreteInstance();

  if(pVVisManager)
  {
    UI->ApplyCommand("/vis/scene/notifyHandlers");
  }
#endif
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void RunAction::EndOfRunAction(const G4Run* /*aRun*/)
{
  NMadoka::Write();
  NMadoka::CloseFile();

#ifdef G4VIS_USE
  if (G4VVisManager::GetConcreteInstance())
    G4UImanager::GetUIpointer()->ApplyCommand("/vis/viewer/update");
#endif
}

void RunAction::SetFileName(G4String val)
{
  fileName = val;
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
